# Generated by Django 2.2.15 on 2021-01-17 18:59

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='GOStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
                ('code', models.CharField(max_length=128)),
                ('parent_obj_description', models.CharField(max_length=128)),
                ('next_statuses', models.CharField(blank=True, max_length=256, null=True)),
                ('is_entrant', models.BooleanField(default=False)),
                ('is_leaving', models.BooleanField(default=False)),
            ],
        ),
    ]



from django.apps import apps
from django.contrib.auth.decorators import login_required
from django.db.models import Q
from django.forms import formset_factory
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.template.loader import render_to_string
from django.urls import reverse

import datetime, importlib, os, sys

from .models import GOStatus
from .utils import generate_xlsx, get_params_for_search, apply_translations, \
        apply_field_updates

# Create your views here.


def general_obj_new(request, Obj, ObjForm, translations={}, \
        callback_prevalidation=False, callback_postvalidation=False, \
        callback_postaction=False, callback_postsaveaction=False, \
        cancel_url=False, return_url=False, html_scripts=False, \
        treat_as_list=False, *args, **kwargs):
    """General form view for adding new objects
    
    It' s not a real view, rather a generic function to creating new objects with the predefined parameters:
    
        - name (the name of the Class)
        - callback_prevalidation (method, runs before form validation)
        - callback_postvalidation (method, runs after form validation)
        - callback_postaction (method, runs after saving the object with commit=False)
        - callback_postsaveaction (method, runs after saving the object)
        - cancel_url (string, in case of cancel, the user will be redirected to here)"""
    ret_dic = {}
    ret_dic['edit'] = False
    ret_dic['class_name'] = Obj._meta.object_name
    field_updates = []
    datalists = []
    if html_scripts:
        ret_dic['html_scripts'] = html_scripts
    if treat_as_list:
        for item in treat_as_list:
            field_name = item['field_name']
            action = item['action']
            if action == 'text_to_choice':
                if 'queryset' in item:
                    datalists.append(item['queryset'])
                else:
                    datalists.append((field_name, Obj.objects.filter().values_list(\
                            field_name, flat=True).distinct().order_by(field_name)))
                field_updates.append((field_name, 'list', field_name))
        if datalists:
            ret_dic['datalists'] = datalists
    if request.method == 'POST':
        cancel = request.POST.get('cancel', '')
        if cancel:
#            request.session['info'] = ('You pressed the Cancel button', \
#            'You got redirected to somewhere')
            if cancel_url:
                return HttpResponseRedirect(cancel_url)
            else:
                if return_url:
                    return HttpResponseRedirect(return_url)
                else:
                    return HttpResponseRedirect(reverse("%ss" % \
                            ret_dic['class_name'].lower()))
        form = apply_translations(ObjForm(request.POST, request=request), \
                translations)
        if callback_prevalidation:
            form = callback_prevalidation(form)
        if form.is_valid():
            if callback_postvalidation:
                form = callback_postvalidation(form)
                if not form.is_valid():
                    ret_dic['form'] = apply_translations(\
                            ObjForm(request.POST, request=request), translations)
                    request.session['err'] = []
                    for field, msg in form._errors.items():
                        request.session['err'].append('%s: %s' % (field, ' | '.join(msg)))
                    return render(request, 'generalobj_new.html', \
                           ret_dic)
            obj = form.save(commit=False)
            if callback_postaction:
                obj = callback_postaction(obj)
            obj.save()
            try:
                form.save_m2m()
            except AttributeError:
                pass
            if callback_postsaveaction:
                obj = callback_postsaveaction(obj, form, request)
                #TBF - it' s an ugly solution.
                #Hint: http://stackoverflow.com/questions/11137309/django-is-it-possible-to-make-a-redirect-from-a-function-in-a-view
                if type(obj) == HttpResponseRedirect:
                    return obj
            request.session['info'] = ['Saved']
            if return_url:
                return HttpResponseRedirect(return_url)
            return HttpResponseRedirect(reverse(ret_dic['class_name'].lower(), \
                    args=(obj.id,)))
        else:
            ret_dic['form'] = apply_field_updates(
                    apply_translations(\
                    ObjForm(request.POST, request=request), translations), \
                    field_updates)
            return render(request, 'generalobj_new.html', \
                    ret_dic)
    else:
        ret_dic['form'] = apply_field_updates(
                apply_translations(ObjForm(request=request), \
                translations), field_updates)
        return render(request, 'generalobj_new.html', ret_dic)

    return HttpResponse('qqq')


def general_obj_edit(request, Obj, ObjForm, general_obj_id, querysets=[], \
        translations={}, callback_prevalidation=False, \
        callback_postvalidation=False, callback_postaction=False, \
        callback_postsaveaction=False, cancel_url=False, return_url=False):
    """General form view for editing a record of an object
    
        It' s not a real view, rather a generic function to edit objects with the predefined parameters:

            - name (the name of the Class)
            - general_obj_id (the id of the object)
            - queryset ()
            - callback_postvalidation (method, runs after form validation)
            - callback_postaction (method, runs after saving the object with commit=False)
            - callback_postsaveaction (method, runs after saving the object)
            - cancel_url (string, in case of cancel, the user will be redirected to here)"""
    ret_dic = {}
    ret_dic['edit'] = True
    ret_dic['class_name'] = Obj._meta.object_name
    obj = get_object_or_404(Obj, id=general_obj_id)
    ret_dic['delete_url'] = os.path.join(reverse(\
            ret_dic['class_name'].lower(), args=(obj.id,)), 'delete/')
    print(ret_dic)
    ret_dic['obj'] = obj
    if request.method == 'POST':
        cancel = request.POST.get('cancel', '')
        if not return_url:
            return_url = request.GET.get('return_url', '')
        if cancel:
#            request.session['info'] = ('You pressed the Cancel button',)
            if cancel_url:
                return HttpResponseRedirect(cancel_url)
            else:
                if return_url:
                    return HttpResponseRedirect(return_url)
                else:
                    return HttpResponseRedirect(reverse(\
                            ret_dic['class_name'].lower(), args=(obj.id,)))
        form = apply_translations(ObjForm(\
                request.POST, instance=obj, request=request), translations)
        if callback_prevalidation:
            form = callback_prevalidation(form)
            if isinstance(obj, HttpResponseRedirect):
                return obj
        if form.is_valid():
            if callback_postvalidation:
                form = callback_postvalidation(form)
                if not form.is_valid():
                    ret_dic['form'] = ObjForm(request.POST, instance=obj, request=request)
                    request.session['err'] = []
                    for field, msg in form._errors.items():
                        request.session['err'].append('%s: %s' % (field, ' | '.join(msg)))
                    return render(request, 'generalobj_new.html', \
                           ret_dic)
            obj = form.save(commit=False)
            if callback_postaction:
                obj = callback_postaction(obj)
            obj.save()
            try:
                form.save_m2m()
            except AttributeError:
                pass
            if callback_postsaveaction:
                obj = callback_postsaveaction(obj)
                if type(obj) == HttpResponseRedirect:
                    return obj
            request.session['info'] = ['Saved']
            if return_url:
                return HttpResponseRedirect(return_url)
            else:
#                return HttpResponseRedirect(reverse("%s_edit" % name.lower(), \
#                        args=(obj.id,)))
                return HttpResponseRedirect(reverse(\
                        ret_dic['class_name'].lower(), args=(obj.id,)))
        else:
            form = ObjForm(request.POST, instance=obj, request=request)
            ret_dic['form'] = apply_translations(form, translations)
            return render(request, 'generalobj_new.html', ret_dic)
    else:
        form = apply_translations(ObjForm(instance=obj, request=request), \
                translations)
        for qs in querysets:
            form.fields[qs[0]].queryset = qs[1]
        ret_dic['form'] = form
        return render(request, 'generalobj_new.html', ret_dic)


def general_obj_bulkedit(request, Obj, ObjForm, general_obj_ids, 
        callback_prevalidation=False, callback_postvalidation=False, \
        callback_postaction=False, callback_postsaveaction=False, \
        cancel_url=False, return_url=False, label=False, \
        editable_fields=False, allow_redirect_to_edit=True):
    ret_dic = {}
    ret_dic['edit'] = True
    ret_dic['class_name'] = Obj._meta.object_name
    ret_dic['label'] = label
    if general_obj_ids == 'all':
        objs = Obj.objects.filter()
    else:
        objs = Obj.objects.filter(id__in = general_obj_ids.split(','))
    #delete url
    ret_dic['objs'] = objs
    if len(objs) > 1:
        pass
    elif len(objs) == 1:
        if allow_redirect_to_edit:
            return HttpResponseRedirect(reverse(\
                    '%s_edit' % ret_dic['class_name'].lower(), \
                    args=(objs[0].id,)))
    else:
        pass
    ret_dic['ids_from_parameter'] = general_obj_ids
    if request.method == 'POST':
        cancel = request.POST.get('cancel', '')
        return_url = request.GET.get('return_url', '')
        if cancel:
            if cancel_url:
                return HttpResponseRedirect(cancel_url)
            else:
                if return_url:
                    return HttpResponseRedirect(return_url)
                else:
                    return HttpResponseRedirect(\
                            reverse('%ss' % ret_dic['class_name'].lower()))
        ret_dic['forms'] = []
        for obj in objs:
            form = ObjForm(request.POST, instance=obj, request=request, \
                    prefix='%s' % obj.id)
            if callback_prevalidation:
                form = callback_prevalidation(form)
                #We don' t want any redirection on a single form action
            obj_can_be_saved = True
            if form.is_valid():
                if callback_postvalidation:
                    form = callback_postvalidation(form)
                    if not form.is_valid():
                        obj_can_be_saved = False
                        #error handling should come here
                if obj_can_be_saved:
                    obj = form.save(commit=False)
                    if callback_postaction:
                        obj = callback_postaction(obj)
                    obj.save()
                    try:
                        form.save_m2m()
                    except AttributeError:
                        pass
                    if callback_postsaveaction:
                        obj = callback_postsaveaction(obj)
            else:
                print('x')
                pass
            formagain = ObjForm(request.POST, instance=obj, request=request, \
                    prefix='%s' % obj.id)
            ctrl = {}
            if hasattr(form, 'label_per_instance'):
                label_per_instance = form.label_per_instance
            else:
                label_per_instance = obj.__str__()
            ctrl['label_per_instance'] = label_per_instance
            ret_dic['forms'].append((formagain, ctrl))
        return render(request, 'generalobj_bulkedit.html', ret_dic)
    else:
        forms = []
        for obj in objs:
            form = ObjForm(instance=obj, request=request, prefix='%s' % obj.id)
            ctrl = {}
            if hasattr(form, 'label_per_instance'):
                label_per_instance = form.label_per_instance
            else:
                label_per_instance = obj.__str__()
            ctrl['label_per_instance'] = label_per_instance
            forms.append((form, ctrl))
        ret_dic['forms'] = (forms)
        return render(request, 'generalobj_bulkedit.html', ret_dic)
                
        
    

def general_obj_delete(request, name, general_obj_id):
    try:
        Obj = eval(name)
    except NameError:
        pass
    try:
        obj = Obj.objects.get(id=general_obj_id)
        obj.delete()
        request.session['info'] = ('%s#%s has been deleted: %s' % \
                (name, general_obj_id, obj.__dict__),)
    except:
        request.session['err'] = ('%s#%s is not deletable: %s' % \
                (name, general_obj_id, sys.exc_info()[0]),)
    return HttpResponseRedirect(reverse('index'))


def general_objs(request, Obj, ObjForm, fields=(), query_dict={}, query_term='', \
        excluded_query_dict={}, no_id_in_fields=False, \
        show_unused_download_excelsheet_button=False, \
        translations_field_names={}, *args, **kwargs):
    """General view for displaying records of an object
    
    It' s not a real view, rather a generic function to displaying filtered list of an Object with a datatable.
    
    The parameters are:
    
        - name (the name of the Class)
        - fields (the fields to be displayed [overrides the default values])
        - query_dict (dictionary for **query)
        - query_term (term for query)
        - excluded_query_dict (dictionary for .exclude(**excluded_query_dict))"""
    ret_dic = {}
    ret_dic['class_name'] = Obj._meta.object_name
    ret_dic['show_unused_download_excelsheet_button'] = \
            show_unused_download_excelsheet_button
    if not fields and not fields == 'DONTDOIT':
        field_names = [field.name for field in Obj._meta.fields]
    else:
        field_names = ['id'] + fields
    if no_id_in_fields:
        field_names.remove('id')
    objs = Obj.objects.filter(**query_dict).exclude(**excluded_query_dict)
    if query_term:
        objs = objs.filter(query_term)
    objs = objs.distinct()
    ret_dic['all_id'] = ','.join([str(obj.id) for obj in objs])
    ret_dic['obj_name'] = ret_dic['class_name']
    fields = []
    obj_name_html = False
    obj_name_html_path = os.path.join(Obj._meta.app_config.path, 'templates', \
            'blocks', '%s_name.html' % ret_dic['class_name'].lower())
    if os.access(obj_name_html_path, os.R_OK):
        obj_name_html = obj_name_html_path
    field_htmls = {}
    for field_name in field_names:
        _field_html = os.path.join(Obj._meta.app_config.path, 'templates', \
                'blocks', '%s_name.html' % field_name)
        if os.access(_field_html, os.R_OK):
            field_htmls[field_name] = _field_html
    for obj in objs:
        if no_id_in_fields:
            field = []
        else:
            if obj_name_html:
                field = [render_to_string(obj_name_html, {ret_dic['obj_name'].lower(): obj})]
            else:
                field = ["<A href='%s'>%s</A>" % (\
                        reverse(ret_dic['class_name'].lower(), \
                        args=(obj.id,)), obj.id)]
        for field_name in field_names:
            if not field_name == 'id':
                if not field_name in field_htmls:
                    field.append(getattr(obj, field_name))
                else:
                    field.append(render_to_string(field_htmls[field_name], {ret_dic['obj_name'].lower(): obj}))
        fields.append(field)
    field_names = [translations_field_names[fn] if fn in translations_field_names else fn for fn in field_names]
    ret_dic['field_names'] = field_names
    ret_dic['fields'] = fields
    ret_dic['objs'] = Obj.objects.filter()
    if not no_id_in_fields:
        ret_dic['url_new'] = reverse('%s_new' % ret_dic['class_name'].lower())
    ret_dic['url_tag'] = ret_dic['class_name'].lower()
    if 'additional_template' in kwargs:
        ret_dic['additional_template'] = kwargs['additional_template']
    if 'pass_to_template' in kwargs:
        for name, value in kwargs['pass_to_template'].items():
            ret_dic[name] = value
    return render(request, 'generalobjs.html', ret_dic)


def general_obj(request, Obj, ObjForm, general_obj_id, fields=(), \
        template=False, html_scripts=False, translations={}, \
        additional_buttons='', status_button_obj=False, *args, **kwargs):
    """General view for displaying a detailed view of an object"""
    if not template:
        template = 'generalobj.html'
    ret_dic = {}
    if html_scripts:
        ret_dic['html_scripts'] = html_scripts
    obj = get_object_or_404(Obj, id=general_obj_id)
    ret_dic['obj'] = obj
    ret_dic['class_name'] = Obj._meta.object_name
    ret_dic['additional_buttons'] = additional_buttons
    if status_button_obj:
        ret_dic['current_status'] = status_button_obj.status
        status_buttons = status_button_obj.status.get_next_statuses()
        ret_dic['status_buttons'] = status_buttons
        ret_dic['parent_obj_id'] = status_button_obj.id
    if not fields:
        fields = [field.name for field in obj._meta.fields]
    if fields:
        ret_dic['fields'] = []
        for field in fields:
            field_name = field
            if field in translations:
                field_name = translations[field]
            ret_dic['fields'].append((field_name, getattr(obj, field)))
    print(ret_dic['fields'])
    ret_dic['url_edit'] = reverse('%s_edit' % ret_dic['class_name'].lower(), \
            args=(obj.id,))
    if 'additional_tabs' in kwargs:
        ret_dic['additional_tabs'] = kwargs['additional_tabs']
    if 'pass_to_template' in kwargs:
        for name, value in kwargs['pass_to_template'].items():
            ret_dic[name] = value
    return render(request, template, ret_dic)


def general_obj_search(request, Obj, ObjForm, search_fields=(), \
        result_fields=(), template=False, allow_prompt_ajax_search=False, \
        allow_overall_search=False, relational_obj_values={}, translations={}, \
        import_instruction_for_obj=False, import_instruction_for_report=False, \
        ajax_url=False, *args, **kwargs):
    if not template:
        template = 'generalobj_search.html'
    ret_dic = {}
    (ret_dic['fields'], ret_dic['class_name'], ret_dic['class_name_lower'], \
            ret_dic['ajax_url'], ret_dic['charfield_exists'], \
            ret_dic['datetimefield_exists'], ret_dic['booleanfield_exists']) = \
            get_params_for_search(request, Obj, ObjForm, search_fields, \
            result_fields, relational_obj_values, translations)
    if ajax_url:
        ret_dic['ajax_url'] = ajax_url
    if 'passing_scripts' in kwargs:
        ret_dic['passing_scripts'] = kwargs['passing_scripts']
    if 'passing_scriptsready' in kwargs:
        ret_dic['passing_scriptsready'] = kwargs['passing_scriptsready']
    ret_dic['url_new'] = reverse('%s_new' % ret_dic['class_name'].lower())
    ret_dic['allow_prompt_ajax_search'] = allow_prompt_ajax_search
    ret_dic['allow_overall_search'] = allow_overall_search
    ret_dic['import_instruction_for_obj'] = import_instruction_for_obj
    ret_dic['import_instruction_for_report'] = import_instruction_for_report
    return render(request, template, ret_dic)


def ajax_get_general_obj_list(request, Obj, ObjForm, search_fields=(), \
        result_fields=(), translations={}, extra_filters={}):
    is_overall_search = request.GET.get('is_overall_search', '')
    (fields, class_name, class_name_lower, ajax_url, charfield_exists, \
            datetimefield_exists, booleanfield_exists) = \
            get_params_for_search(request, Obj, ObjForm, search_fields, \
            result_fields)
    print(request.GET)
    class_names_lower = '%ss' % class_name_lower
    

    ret_dic = {}
    if not result_fields:
        result_fields = [field.name for field in Obj._meta.fields]

    if is_overall_search:
        overall_search = request.GET.get('overall_search', '')
        q = Q()
        for field in fields:
            if field[1] == 'text':
                q = q|Q(("%s__icontains" % field[0], overall_search))
        ret_dic['objects'] = Obj.objects.filter(q)
    else:
        get_fields = {}
        print(fields)
        for field in fields:
            if field[2] == 'ltgt':
                get_fields['%s_gt' % field[0]] = request.GET.get(\
                        '%s_gt' % field[0], '')
                get_fields['%s_lt' % field[0]] = request.GET.get(\
                        '%s_lt' % field[0], '')
            elif field[2] == 'text':
                get_fields[field[0]] = request.GET.get(field[0], '').strip()
            elif field[2] == 'relation':
                get_fields[field[0]] = \
                        request.GET.get(field[0], '').strip().split(',')
        print('gf = ', get_fields)
        useful_data_available = False
        useful_data_available = any([item[1] for item in get_fields.items()])
        print('uf = ', useful_data_available)
        if useful_data_available:
            dt = {}
            for field in fields:
                if field[1] == 'number':
                    if get_fields['%s_gt' % field[0]]:
                        dt['%s__gte' % field[0]] = get_fields['%s_gt' % field[0]]
                    if get_fields['%s_lt' % field[0]]:
                        dt['%s__lte' % field[0]] = get_fields['%s_lt' % field[0]]
                elif field[1] == 'text':
                    if get_fields[field[0]]:
                        dt['%s__icontains' % field[0]] = \
                                get_fields[field[0]].strip()
                elif field[1] == 'date':
                    if get_fields['%s_gt' % field[0]]:
                        dt['%s__gte' % field[0]] = datetime.datetime.strptime(\
                                get_fields['%s_gt' % field[0]], '%d/%m/%Y')
                    if get_fields['%s_lt' % field[0]]:
                        dt['%s__lte' % field[0]] = datetime.datetime.strptime(\
                                get_fields['%s_lt' % field[0]], '%d/%m/%Y')
                elif field[1] == 'boolean':
                    if get_fields[field[0]]:
                        TRUE = 'true' in get_fields[field[0]].split(',')
                        FALSE = 'false' in get_fields[field[0]].split(',')
                        if TRUE and FALSE:
                            pass
                        elif TRUE:
                            dt[field[0]] = True
                        else:
                            dt[field[0]] = False
                elif field[1] == 'relation':
                    if any(get_fields[field[0]]):
                        if 'modificator' in field[3]:
                            dt['%s__in' % field[0]] = get_fields[field[0]]
                        else:
                            dt['%s__id__in' % field[0]] = get_fields[field[0]]
            ret_dic['objects'] = Obj.objects.filter(**dt)
            print('dt = %s' % dt)
        else:
            ret_dic['objects'] = Obj.objects.all()
    if extra_filters:
        ret_dic['objects'] = ret_dic['objects'].filter(**extra_filters)
    print(request.GET)
    ret_dic['all_id'] = ','.join([str(obj.id) for obj in \
            ret_dic['objects']])
    ret_dic['result_fields'] = result_fields
    ret_dic['field_names'] = [translations[fn] if fn in translations else fn for fn in result_fields]
    return render(request, 'ajax/get_object_list.html', ret_dic)
    print(dt)
    print('xx', useful_data_available)


def download_excelsheet(request):
    import_instruction_for_obj = \
            request.GET.get('ge_import_instruction_for_obj', '')
    import_instruction_for_report = \
            request.GET.get('ge_import_instruction_for_report', '')
    ge_ids = request.GET.get('ge_ids', '')

    (_model, _cls) = import_instruction_for_obj.split('.')
    Obj = apps.get_model(_model, _cls)
    obj_list = locals()['Obj'].objects.filter(id__in=ge_ids.split(','))

    (_modul, _file, _var, _key) = import_instruction_for_report.split('|')
    module = importlib.import_module('%s.%s' % (_modul, _file))
    reports = getattr(module, _var)
    report = reports[_key]

    filename = generate_xlsx(obj_list, report)
    download = open(filename, 'rb')
    response = HttpResponse(download.read())
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    return response


def set_gostatus(request):
    #If everything passes, new status will be set, and redirection is performed
    #to the return_url which is calculated automatically from request.path
    current_status = request.POST.get('current_status', '')
    new_status = request.POST.get('new_status', '')
    cs_id = current_status.split('_')[0]
    cs_pod = '_'.join(current_status.split('_')[1:])
    parent_obj_id = request.POST.get('parent_obj_id', '')
    return_url = request.POST.get('gos_return_url', '')
    try:
        cs_obj = GOStatus.objects.get(id=cs_id, \
                parent_obj_description=cs_pod)
    except:
        request.session['err'] = ('Error at current status',)
        return HttpResponse(return_url)
    if not new_status in \
            cs_obj.get_next_statuses().values_list('code', flat=True):
        request.session['err'] = ('Wrong new status',)
        print(return_url)
        return HttpResponseRedirect(return_url)
    try:
        ns_obj = GOStatus.objects.get(code=new_status, \
                parent_obj_description=cs_obj.parent_obj_description)
    except:
        request.session['err'] = ('Next status does not exist',)
        return HttpResponse(return_url)
    parent_obj_model, attr_name = cs_obj.get_attr_parts()
    parent_obj = parent_obj_model.objects.get(id=parent_obj_id)
#    parent_obj.status = ns_obj
#    cs_obj.set_status(ns_obj)
    setattr(parent_obj, attr_name, ns_obj)
    parent_obj.save()
    request.session['info'] = ('Status has been updated',)
    return HttpResponseRedirect(return_url)



# Generated by Django 2.2.15 on 2021-01-18 23:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('generalobj', '0002_auto_20210117_2150'),
    ]

    operations = [
        migrations.AddField(
            model_name='gostatus',
            name='is_backward',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='gostatus',
            name='is_forward',
            field=models.BooleanField(default=True),
        ),
    ]



from django.conf.urls import url
from generalobj import views

urlpatterns = [
    url('download_excelsheet', views.download_excelsheet, \
            name='download_excelsheet'),
    url('set_gostatus', views.set_gostatus, name='set_gostatus'),
]


from django.db import models

from django.apps import apps


class GOStatus(models.Model):
    name = models.CharField(max_length=128)
    code = models.CharField(max_length=128)
    parent_obj_description = models.CharField(max_length=128) #'$appname___$modelname
    next_statuses = models.CharField(max_length=256, blank=True, null=True, \
            default='')
    is_entrant = models.BooleanField(default=False)
    is_leaving = models.BooleanField(default=False)
    is_forward = models.BooleanField(default=True)
    is_backward = models.BooleanField(default=False)

    def get_next_statuses(self):
        return GOStatus.objects.filter(\
                parent_obj_description=self.parent_obj_description).filter(\
                code__in=self.next_statuses.split('|'))

    def add_status(self, name):
        if not self.next_statuses:
            self.next_statuses = ''
        if '|%s|' % name in self.next_statuses:
            return True
        else:
            if not self.next_statuses.startswith('|'):
                self.next_statuses = '|%s' % self.next_statuses
            if not self.next_statuses.endswith('|'):
                self.next_statuses += '|'
            self.next_statuses += '%s|' % name
            self.save()

    def remove_status(self, name):
        if '|%s|' % name in self.next_statuses:
            self.next_statuses = self.next_statuses.replace('%s|' % name, '')
            self.save()

    def add_statuses(self, name_lst):
        self.next_statuses = '|%s|' % '|'.join(sorted(name_lst))
        self.save()


    def get_model(self):
        try:
            pob = self.parent_obj_description.split('___')
            model_base = apps.get_model(pob[0], pob[1])
            return model_base
        except:
            return False

    def get_attr_parts(self):
        mdl = self.get_model()
        if mdl:
            try:
                pob = self.parent_obj_description.split('___')
                return (mdl, pob[2])
            except:
                return (mdl, 'status')
        else:
            return False

    def set_status(self, status):
        self.status = status
        self.save()


    def change_parent_obj_status(self, parent_obj_id, ns_code, check_next_statuses=True):
        if check_next_statuses:
            if not '|%s|' % ns_code in self.next_statuses:
                print("Status can't be set. %s is not next." % ns_code)
        ns_obj = GOStatus.objects.get(code=ns_code, \
                parent_obj_description=self.parent_obj_description)
        parent_obj_base, attr_name = self.get_attr_parts()
        parent_obj = parent_obj_base.objects.get(id=parent_obj_id)
        if parent_obj:
            setattr(parent_obj, attr_name, ns_obj)
            parent_obj.save()
            

    def __str__(self):
        return '%s -> %s' % (self.code, self.next_statuses)



# Generated by Django 2.2.15 on 2021-01-17 21:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('generalobj', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gostatus',
            name='next_statuses',
            field=models.CharField(blank=True, default='', max_length=256, null=True),
        ),
    ]



#!/usr/bin/python
# -*- coding: utf-8 -*-

from django.core.management.base import BaseCommand
from optparse import make_option

from generalobj.models import GOStatus

import sys


class Command(BaseCommand):
    
    def add_arguments(self, parser):
        parser.add_argument('--file', action='store', dest='filename', \
                required=True, help='filename for file is required')
        parser.add_argument('--dictionary', action='store', dest='dt', \
                required=True, help='The name of the dictionary from the file')
        #If reset is not specified, it'll do anything only if there're no
        #GOStatuses with the given pod.
        parser.add_argument('--reset', action='store_true', dest='reset', \
                help='If there is any GOStatus, it will delete them first')
        #Mostly it's a ForeignKey, so it wouldn't be a good idea to delete the
        #pks and recreate.
        parser.add_argument('--parent_obj_description', action='store', \
                dest='parent_obj_description', required=True, \
                help="It helps identify which GOS. It won't delete any GOS,\
                but tries to identify by code and update. If no match, it'll \
                create a new one.")

    def handle(self, *args, **options):
        #Get command line parameters
        filename = options['filename']
        dt_name = options['dt']
        reset = options['reset']
        pod = options['parent_obj_description']
        
        #I'll work only with GOStatuses with the given pod
        current_goses = GOStatus.objects.filter(parent_obj_description=pod)
        if not reset and current_goses:
            print("There are GOStatuses, but i can't reset them. Exiting.") 
            sys.exit(1)

        #Reading the config file and the given dictionary. Might be dangerous,
        #maybe i should change and use configparser.
        exec(open(filename).read())
        dt = eval(dt_name)

        #Checking for errors in the dictionary. Code and name are mandatory.
        errors = []
        if not 'pod' in dt:
            errors.append('pod is not in dictionary')
        for item in dt['items']:
            if not 'code' in item.keys():
                errors.append('code is not in item: %s' % item)
            else:
                if not item['code']:
                    errors.append('code is empty in item: %s' % item)
            if not 'name' in item.keys():
                errors.append('name is not in item: %s' % item)
            else:
                if not item['name']:
                    errors.append('name is empty in item: %s' % item)
        #Printing the errors and exiting.
        if errors:
            for count, error in enumerate(errors):
                print('%s.\t%s' % (count, error))
            sys.exit(2)

        #Creating / Updating the GOStatuses based on the dictionary.
        #If updating, the binding argument is the code.
        #If it's a new item, id can be updated.
        for item in dt['items']:
            if reset:
                try:
                    gos = current_goses.get(code=item['code'])
                except:
                    gos = GOStatus(parent_obj_description=pod, \
                            code=item['code'])
                    #If it's a new item, ID might be set
                    if 'id' in item.keys():
                        gos.id = item['id']
            else:
                gos = GOStatus(parent_obj_description=pod, code=item['code'])
                #If it's a new item, ID might be set
                if 'id' in item.keys():
                    gos.id = item['id']
            gos.name = item['name']
            if 'next_statuses' in item.keys():
                gos.next_statuses = item['next_statuses']
            if 'is_entrant' in item.keys():
                gos.is_entrant = item['is_entrant']
            if 'is_leaving' in item.keys():
                gos.is_leaving = item['is_leaving']
            if 'is_forward' in item.keys():
                gos.is_forward = item['is_forward']
            if 'is_backward' in item.keys():
                gos.is_backward = item['is_backward']
            if 'style' in item.keys():
                gos.style = item['style']
            gos.save()



from django import forms
from django.contrib.auth.models import User

class GeneralObjForm(forms.ModelForm):
    """
    This is a general class for forms used almost everywhere.
    
    It sets the autofocus if needed
    
    It handles live_search parameters
    
    It takes care about help_texts
    
    It handles User selections and DatePickers
    
    It makes automatic changes to Labels.
    """
    
    def __init__(self, *args, **kwargs):
        """The init method"""
        self.request = kwargs.pop('request', None)
        super(GeneralObjForm, self).__init__(*args, **kwargs)
        #slow solution, needs to be replaced!
        focus_to = [field for field in self.fields][0]
        self.fields[focus_to].widget.attrs.update({'autofocus': 'autofocus'})
        for field in self.fields:
            sfield = self.fields[field]

            #live_search
            self.fields[field].live_search = False
            if hasattr(sfield, 'choices'):
                if sfield.choices.queryset.count() > 10:
                    sfield.live_search = True

            #Help text
            help_text = self.fields[field].help_text
#            self.fields[field].help_text = None
##            if help_text != '':
##                self.fields[field].widget.attrs.update(\
##                        {'class':'has-popover',
##                         'data-content':help_text,
##                         'data-placement':'right',
##                         'data-container':'body'
##                        })

            #User selections
            if hasattr(sfield, 'choices'):
                if sfield.choices.queryset.count():
                    if type(sfield.choices.queryset[0]) == User:
                        sfield.label_from_instance = lambda obj: "%s %s" % (obj.first_name, obj.last_name)
                        sfield.queryset = sfield.queryset.filter(is_active = True).order_by('first_name')

            #DateField
            sfield.date_field = False
            if hasattr(sfield, 'strptime'):
                sfield.date_field = True
                sfield.input_formats = ['%d/%m/%Y']
                inst = eval('self.instance.%s' % field)
                pass
                if inst:
                    self.initial[field] = inst.strftime('%d/%m/%Y')
                elif sfield.initial:
                    sfield.initial = sfield.initial.strftime('%d/%m/%Y')
            
            #User -> Creator
            if field == 'user':
                sfield.label = 'Creator'
    
    
            #Checkbox
            sfield.checkbox_field = False
            if sfield.__class__.__name__ == 'BooleanField':
                sfield.checkbox_field = True



from django.urls import reverse
from operator import attrgetter

import datetime
import xlsxwriter

def get_params_for_search(request, Obj, ObjForm, search_fields, result_fields, \
        relational_obj_values={}, translations={}):
    print('trk = ', translations)
    if not search_fields:
        search_fields = [field.name for field in Obj._meta.fields]
    fields = []
    charfield_exists = False
    datetimefield_exists = False
    booleanfield_exists = False
    for field in search_fields:
        _internal_type = False
        this_is_a_modificator = False
        if type(field) == tuple:
            this_is_a_modificator = {'treat_as': 'in'}
            (field, _internal_type) = field
            if not field in relational_obj_values:
                relational_obj_values[field] = Obj.objects.filter().values_list(field, field).distinct().order_by(field)
        remaining = {}
        remaining['label'] = field
        if this_is_a_modificator:
            remaining['modificator'] = this_is_a_modificator
        if field in translations:
            remaining['label'] = translations[field]
        if _internal_type:
            internal_type = _internal_type
        else:
            internal_type = Obj._meta.get_field(field).get_internal_type()
        print(internal_type)
        if internal_type in ('IntegerField', 'FloatField', 'DecimalField'):
            fields.append((field, 'number', 'ltgt', remaining))
            charfield_exists = True
        elif internal_type in ('CharField', 'TextField'):
            fields.append((field, 'text', 'text', remaining))
            charfield_exists = True
        elif internal_type in ('DateField', 'DateTimeField'):
            fields.append((field, 'date', 'ltgt', remaining))
            datetimefield_exists = True
        elif internal_type == 'ForeignKey':
            values = []
            if field in relational_obj_values:
                remaining['relational_obj_values'] = relational_obj_values[field]
            fields.append((field, 'relation', 'relation', remaining))
        elif internal_type == 'BooleanField':
            fields.append((field, 'boolean', 'text', remaining))
            booleanfield_exists = True
    print(fields)
    class_name = Obj._meta.object_name
    class_name_lower = class_name.lower()
    ajax_url = reverse('ajax_get_%s_list' % class_name.lower())
    print(fields, class_name, class_name_lower, ajax_url, charfield_exists, \
            datetimefield_exists, booleanfield_exists)
    return (fields, class_name, class_name_lower, ajax_url, charfield_exists, \
            datetimefield_exists, booleanfield_exists)


def generate_xlsx(object_list, REPORT, filename=False):
    data = []
    for obj in object_list:
        data_inline = []
#        if not report_name:
#            report_name = obj.__class__.__name__
        class_name = obj.__class__.__name__
        if 'parameters' in REPORT:
            for attr in REPORT['parameters']:
                _value = attrgetter(attr)(obj)
                if _value == None:
                    _value = ''
                data_inline.append(_value)
            data.append(data_inline)
    print(data)

    if not filename:
        now = datetime.datetime.now()
        filename = '/tmp/generate_xlsx_%s_%s.xlsx' % (class_name, now.strftime('%Y_%m_%d__%H_%M_%S'))

    wb = xlsxwriter.Workbook(filename)
    ws = wb.add_worksheet()

    row = 0
    col = 0
    try:
        _header = REPORT['header']
    except:
        _header = REPORT['parameters']
    for attr in _header:
        ws.write(row, col, str(attr)[:1].upper() + str(attr)[1:])
        col = col + 1

    row = 1
    for line in data:
        col = 0
        for item in line:
            ws.write(row, col, str(item))
            col = col + 1
        row = row + 1
    wb.close()

    return filename


def apply_translations(form, translations=False):
    if translations:
        for field in form.fields:
            sfield = form.fields[field]
            if field in translations:
                sfield.label = translations[field]
        return form
    return form

def apply_field_updates(form, lst):
    if lst:
        for field in form.fields:
            sfield = form.fields[field]
            for field_name, widget_name, attr in lst:
                if field == field_name:
                    sfield.widget.attrs.update({widget_name: attr})
    return form
        
        


from django import template

import math

register = template.Library()

#def cut(value, arg):
#    return value.replace(arg, '')

def as_html(obj):
    """A helper function for the admin panel"""
    if hasattr(obj, 'username'):
        return obj.username
    if hasattr(obj, 'as_html'):
        print("xxx")
        return obj.as_html()
    if hasattr(obj, 'name'):
        return obj.name
    return "I don' t know"

register.filter('as_html', as_html)

#add more than one and include old classes
@register.filter(name='addattr')
def addattr(field, css):
    """Adds the specified attributes to the html"""
    attrs = {}
    definition = css.split(',')
    if field.field.widget.attrs.get('class', None):
        classes = field.field.widget.attrs.get('class', None)
    else:
        classes = ''

    for d in definition:
        t, v = d.split(':')
        if t == 'class':
            classes += ' ' + v
        else:
            attrs[t] = v

    attrs['class'] = classes

    return field.as_widget(attrs=attrs)

#keep old classes too
#def addcss(field, css):
#class_old = field.field.widget.attrs.get('class', None)
#class_new = class_old + ' ' + css if class_old else css
#return field.as_widget(attrs={"class": class_new})

#def addcss(field, css):
   #return field.as_widget(attrs={"class":css})

@register.filter(name='as_service_level')
def as_service_level(field):
    """It changes the service_level (number) to text"""
    field = int(field)
    if field == 0:
        return 'None'
    elif field == 1:
        return 'Bronze'
    elif field == 2:
        return 'Silver'
    elif field == 3:
        return 'Gold'
    else:
        return 'Unknown'


@register.filter(name="date_format")
def date_format(datum):
    """Sets english date format"""
    if datum:
        return datum.strftime('%d/%m/%Y')
    return datum


@register.filter(name='ago')
def ago(second):
    try:
        second = int(second)
    except ValueError:
        return "<I>%s</I>" % second
    years = math.floor(second / (365 * 86400))
    remaining = second % (365 * 86400)
    days = math.floor(remaining / (86400))
    remaining = second % 86400
    hours = math.floor(remaining / 3600)
    remaining = second % 3600
    minutes = math.floor(remaining / 60)
    seconds = second % 60
    ret_str = ''
    if years:
        ret_str += '<font color="red">%s</font>y' % years
    if days:
        ret_str += ' <font color="blue">%s</font>d' % days
    if hours:
        ret_str += ' <font color="green">%s</font>h' % hours
#    if minutes:
#        ret_str += ' <font color="green">%s</font>m' % minutes
#    if seconds:
#        ret_str += ' <font color="r">%s</font>s' % seconds
    return ret_str


@register.filter(name='template_exists')
def template_exists(template_name):
    try:
        template.loader.get_template(template_name)
        return True
    except template.TemplateDoesNotExist:
        return False

@register.filter(name='ifinlist')
def ifinlist(value, lst):
    """Checks if string is in a comma separated list"""
    return value in lst.split(',')


@register.filter(name='getta')
def getta(obj, arg):
    if hasattr(obj, str(arg)):
        return getattr(obj, arg)
    else:
        return 'Error fetching the attribute'



from setuptools import setup


long_description = open('README.md', 'r').read()

setup(name='generalobj',
    version='0.10.14',
    description='Generalobj is a package for easy form handling for general Form Objects',
    url='',
    author='Csaba Saranszky',
    author_email='alt@256.hu',
    license='GPL',
    packages=['generalobj'],
    install_requires=['XlsxWriter'],
    zip_safe=False,
    include_package_data=True,
    long_description=long_description,
    long_description_content_type='text/markdown')
